#!/usr/bin/perl -w
use v5.14;
use strict;
use warnings;

use Gruntmaster::Daemon;

use Cwd qw/cwd/;
use File::Copy qw/copy/;
use File::Copy::Recursive qw/dircopy/;
use File::Path qw/remove_tree/;
use File::Temp qw/tempdir/;
use List::Util qw/sum/;
use Log::Log4perl;
use Test::More;
use YAML::Any qw/LoadFile/;

##################################################

my $loglevel = $ENV{TEST_LOG_LEVEL} // 'OFF';
my $log_conf = <<CONF;
log4perl.category.Gruntmaster.Daemon = $loglevel, stderr

log4perl.appender.stderr                          = Log::Log4perl::Appender::Screen
log4perl.appender.stderr.layout                   = Log::Log4perl::Layout::PatternLayout
log4perl.appender.stderr.layout.ConversionPattern = [\%d] [\%F{1}:\%M{1}:\%L] [\%p] \%m\%n
CONF
Log::Log4perl->init(\$log_conf);

$ENV{PATH}.=':' . cwd;

sub check_job{
  my $job = shift;
  my $meta = LoadFile "$job/meta.yml";
  if (exists $meta->{results}) {
	delete $meta->{results}[$_]{time} for keys $meta->{results};
  }
  is $meta->{result}, $meta->{expected_result}, "Result is correct";
  is $meta->{result_text}, $meta->{expected_result_text}, "Result text is correct";
  is_deeply $meta->{results}, $meta->{expected_results}, "Results are correct";
}

my @problems = exists $ENV{TEST_PROBLEMS} ? map {"t/problems/$_"} split ' ', $ENV{TEST_PROBLEMS} : <t/problems/*>;
plan tests => 3 * sum map { my @temp = <$_/tests/*>; scalar @temp } @problems;
note "Problems to be tested: " . join ', ', @problems;

my $tempdir = tempdir CLEANUP => 1;
mkdir "$tempdir/jobs";
dircopy 't/problems' => "$tempdir/pb";

for my $problem (@problems) {
  my $meta = LoadFile "$problem/meta.yml";
 TODO: {
	local $TODO = $meta->{todo} if exists $meta->{todo};
	note "Now testing problem $meta->{name} ($meta->{description})";
	for my $source (<$problem/tests/*>) {
	  my $meta = LoadFile "$source/meta.yml";
	  note "Running $meta->{test_name} ($meta->{test_description})...";
	  dircopy $source => "$tempdir/jobs/job";
	  my $savedcwd = cwd;
	  Gruntmaster::Daemon::process "$tempdir/jobs/job";
	  check_job "$tempdir/log/1";
	  chdir $savedcwd;
	  remove_tree "$tempdir/log", "$tempdir/jobs", {keep_root => 1};
	}
  }
}
